#include <stdafx.h>
#include "joystick.h"
#include <string>
#include <windows.h>

using namespace std;

BOOL CALLBACK EnumObjectsCallback( const DIDEVICEOBJECTINSTANCE* pdidoi, VOID* pContext );
BOOL CALLBACK EnumJoysticksCallback( const DIDEVICEINSTANCE* pdidInstance, VOID* pContext );

#define SAFE_DELETE(p)  { if(p) { delete (p);     (p)=NULL; } }
#define SAFE_RELEASE(p) { if(p) { (p)->Release(); (p)=NULL; } }

LPDIRECTINPUT8 g_pDI = NULL;     
LPDIRECTINPUTDEVICE8 g_pJoystick = NULL;  
HRESULT     hr;
DIJOYSTATE2 js;
const DIDEVICEINSTANCE* joy1;

joystick::joystick(void){
	//initalize values
	button1 = 0;
	button2 = 0;
	button3 = 0;
	button4 = 0;
	button5 = 0;
	button6 = 0;
	button7 = 0;
	button8 = 0;
	button9 = 0;
	button10 = 0;
	button11 = 0;
	button12 = 0;
	button13 = 0;
	button14 = 0;
	button15 = 0;
	button16 = 0;

	x1 = 0;
	x2 = 0;
	x3 = 0;

	rotation1 = 0;
	rotation2 = 0;
	rotation3 = 0;

	slider1 = 0;
	slider2 = 0;
}

joystick::~joystick(void){
	FreeDirectInput();
}


void joystick::FreeDirectInput() {
	if( g_pJoystick ) {
		g_pJoystick->Unacquire();
	}
    
	SAFE_RELEASE( g_pJoystick );
	SAFE_RELEASE( g_pDI );
}


BOOL CALLBACK EnumJoysticksCallback( const DIDEVICEINSTANCE* pdidInstance,VOID* pContext ) {
    UNREFERENCED_PARAMETER( pContext );
    HRESULT hr;

    // Obtain an interface to the enumerated joysticks
	hr = g_pDI->CreateDevice( pdidInstance->guidInstance, &g_pJoystick, NULL );

    // If it failed, then we can't use this joystick. (Maybe the user unplugged
    // it while we were in the middle of enumerating it.)
    if( FAILED(hr) ) 
        return DIENUM_CONTINUE;
 
    // Stop enumeration. Note: we're just taking the first joystick we get. You
    // could store all the enumerated joysticks and let the user pick.

    return DIENUM_STOP;
}



bool joystick::initDevice() {
    // Register with the DirectInput subsystem and get a pointer
    // to a IDirectInput interface we can use.
    // Create a DInput object
    DirectInput8Create( GetModuleHandle(NULL), DIRECTINPUT_VERSION, IID_IDirectInput8, (VOID**)&g_pDI, NULL );

    // Look for a simple joystick we can use for this sample program.
	g_pDI->EnumDevices( DI8DEVCLASS_GAMECTRL, EnumJoysticksCallback,NULL, DIEDFL_ATTACHEDONLY );

    //Check to see if 0 js was found
    if( NULL == g_pJoystick ) {
		return false;
    }

    // Set the data format to "simple joystick" - a predefined data format 
    //
    // A data format specifies which controls on a device we are interested in,
    // and how they should be reported. This tells DInput that we will be
    // passing a DIJOYSTATE2 structure to IDirectInputDevice::GetDeviceState().
    g_pJoystick->SetDataFormat( &c_dfDIJoystick2 );
	
	return true;
}


bool joystick::updateVars() {
	if( NULL == g_pJoystick ) {
        return NULL;
	}

    // Poll the device to read the current state
    hr = g_pJoystick->Poll(); 
    if( FAILED(hr) )  
    {
        // DInput is telling us that the input stream has been
        // interrupted. We aren't tracking any state between polls, so
        // we don't have any special reset that needs to be done. We
        // just re-acquire and try again.
        hr = g_pJoystick->Acquire();
		
		while( hr == DIERR_INPUTLOST ) {
            hr = g_pJoystick->Acquire();
		}

        // hr may be DIERR_OTHERAPPHASPRIO or other errors.  This
        // may occur when the app is minimized or in the process of 
        // switching, so just try again later 
        return true; 
    }

    // Get the input's device state
	if( FAILED( hr = g_pJoystick->GetDeviceState( sizeof(DIJOYSTATE2), &js ) ) ) {
        return false; // The device should have been acquired during the Poll()
	}

	//initalize button values to 0
	button1 = 0;
	button2 = 0;
	button3 = 0;
	button4 = 0;
	button5 = 0;
	button6 = 0;
	button7 = 0;
	button8 = 0;
	button9 = 0;
	button10 = 0;
	button11 = 0;
	button12 = 0;
	button13 = 0;
	button14 = 0;
	button15 = 0;
	button16 = 0;

    for( int i = 0; i < 128; i++ ){
		if(js.rgbButtons[i] & 0x80){
			if(i == 0){
				button1 = 1;
			}
			if(i == 1){
				button2 = 1;
			}
			if(i == 2 ){
				button3 = 1;
			}
			if(i == 3 ){
				button4 = 1;
			}	
			if(i == 4 ){
				button5 = 1;
			}	
			if(i == 5 ){
				button6 = 1;
			}	
			if(i == 6 ){
				button7 = 1;
			}	
			if(i == 7 ){
				button8 = 1;
			}	
			if(i == 8 ){
				button9 = 1;
			}	
			if(i == 9 ){
				button10 = 1;
			}	
			if(i == 10 ){
				button11 = 1;
			}	
			if(i == 11 ){
				button12 = 1;
			}	
			if(i == 12 ){
				button13 = 1;
			}	
			if(i == 13 ){
				button14 = 1;
			}	
			if(i == 14 ){
				button15 = 1;
			}	
			if(i == 15 ){
				button16 = 1;
			}	
		}
	}

	// Axes
	x1 = js.lX;
	x2 = js.lY;
	x3 = js.lZ;

	//Rotational
	rotation1 = js.lRx;
	rotation2 = js.lRy;
	rotation3 = js.lRz;

    //Slider controls
	slider1 = js.rglSlider[0];
	slider2 = js.rglSlider[1];

//Not used in this demo
/*
    //Direction/Point of view
	pov1 = js.rgdwPOV[0];
	pov2 = js.rgdwPOV[1];
	pov3 = js.rgdwPOV[2];
	pov4 = js.rgdwPOV[3];    

	//Angular acceleration
	angAccelX = js.lARx;
	angAccelY = js.lARy;
	angAccelZ = js.lARz;

	//Acceleration
	accelX = js.lAX;
	accelY = js.lAY;
	accelZ = js.lAZ;

	//Torque
	torqueX = js.lFRx;
	torqueY = js.lFRy;
	torqueZ = js.lFRz;

	//Force
	forceX = js.lFX;
	forceY = js.lFY;
	forceZ = js.lFZ;

	//Angular Velocity
	angVelX = js.lVRx;
	angVelY = js.lVRy;
	angVelZ = js.lVRz;

	//Velocity
	velX = js.lVX;
	velY = js.lVY;
	velZ = js.lVZ;

	//Acceleration
	extraAccelA = js.rglASlider[0];
	extraAccelB = js.rglASlider[1];

	//Force
	extraForceA = js.rglFSlider[0];
	extraForceB = js.rglFSlider[1];

	//Velocity
	extraVelA = js.rglVSlider[0];
	extraVelB = js.rglVSlider[1];
*/
	return true;
}